#pragma region Copyright 2012 IPETRONIK GmbH & Co.KG
//
// All rights are reserved. Reproduction or transmission in whole or in part, in
// any form or by any means, electronic, mechanical or otherwise, is prohibited
// without the prior written consent of the copyright owner.
//
// KGI
//
#pragma endregion

///////////////////////////////////////////////////////////////////////////////////////////
/// <summary>
/// include
/// </summary>
#include "Extension.h"

///////////////////////////////////////////////////////////////////////////////////////////
/// <summary>
/// lib
/// </summary>
#pragma comment(lib, "Version.lib")

///////////////////////////////////////////////////////////////////////////////////////////
/// <summary>
/// Global member: callback functions
/// </summary>
PFN_EXT_SET_PACKET_DATA_FLOAT64_FUNC g_pfnDataFloat64 = NULL;      // transfer packet data of double type to the plugin
PFN_EXT_SEND_DATA_FUNC               g_pfnSendData = NULL;         // send data via the systems COM port
PFN_EXT_READ_INPUT_BUFFER            g_pfnReadInputBuffer = NULL;  // read new data received via the
                                                                   // systems COM port since last call of g_pfnReadInputBuffer
PFN_EXT_RESET_INPUT_BUFFER           g_pfnResetInputBuffer = NULL; // reset the input buffer of the systems COM port

PFN_EXT_TRACE_MESSAGE                g_pfnTraceInformation = NULL; // Set a information message to the trace log. Message is
                                                                   // a null terminated WCHAR string with maximum length of MAX_WCHAR (currently 256)
PFN_EXT_TRACE_MESSAGE                g_pfnTraceWarning = NULL;     // Set a warning message to the trace log. Message is
                                                                   // a null terminated WCHAR string with maximum length of MAX_WCHAR (currently 256)
PFN_EXT_TRACE_MESSAGE                g_pfnTraceError = NULL;       // Set a error message to the trace log. Message is
                                                                   // a null terminated WCHAR string with maximum length of MAX_WCHAR (currently 256)

#ifdef __cplusplus
extern "C" {
#endif

///////////////////////////////////////////////////////////////////////////////////////////
/// <summary>
/// Called when the driver is being initialized. This function is called once at the end of the PlugIn function "InitDriver".
/// InitDriver is called directly after SetCallback.
/// </summary>
/// <param name="pwszErrorMessage">
///   reserved for future use - 
///   pointer to buffer for an error message, propagated to
///   IPEmotion when function returns E_FAIL
/// </param>
HRESULT EXTENSION_API InitDriver(WCHAR* pwszErrorMessage)
{
	HRESULT RetVal = NO_ERROR;

	// place optional initializing code here

	return RetVal;
}

///////////////////////////////////////////////////////////////////////////////////////////
/// <summary>
/// Called when exiting driver. This function is called once at the beginning of the PlugIn function "ExitDriver".
/// </summary>
/// <param name="pwszErrorMessage">
///   reserved for future use - 
///   pointer to buffer for an error message, propagated to
///   IPEmotion when function returns E_FAIL
/// </param>
HRESULT EXTENSION_API ExitDriver(WCHAR* pwszErrorMessage)
{
	HRESULT RetVal = NO_ERROR;

	// Place optional uninitializing code here. E.g.:
	// - free memory previously allocated with new or malloc
	// - release resources if any have been reserved
	// - close any opened files
	// - close any opened drivers
	// - ...
	// If this is not done (if needed) the dll might not unload when IPEmotioin is being closed
	// such that it appears that IPEmotion or the Plugin hangs up.

	return RetVal;
}

///////////////////////////////////////////////////////////////////////////////////////////
/// <summary>
/// Assignment of callback functions. This function is called once at the end of the PlugIn function "InitDriver".
/// SetCallbacks is called just before InitDriver.
/// </summary>
HRESULT EXTENSION_API SetCallbacks(PFN_EXT_SET_PACKET_DATA_FLOAT64_FUNC pfnDataFloat64,
                                   PFN_EXT_SEND_DATA_FUNC               pfnSendData,
                                   PFN_EXT_READ_INPUT_BUFFER            pfnReadInputBuffer,
                                   PFN_EXT_RESET_INPUT_BUFFER           pfnResetInputBuffer,
                                   PFN_EXT_TRACE_MESSAGE                pfnTraceInformation,
                                   PFN_EXT_TRACE_MESSAGE                pfnTraceWarning,
                                   PFN_EXT_TRACE_MESSAGE                pfnTraceError)
{
	HRESULT RetVal = NO_ERROR;

	// Set callback functions
	// This call is mandatory.
	RetVal = DoSetCallbacks(pfnDataFloat64, pfnSendData, pfnReadInputBuffer, pfnResetInputBuffer,
							pfnTraceInformation, pfnTraceWarning, pfnTraceError);

	return RetVal;
}

///////////////////////////////////////////////////////////////////////////////////////////
/// <summary>
/// Measurement preparation. This function is called once for every System in the PlugIn function "PrepareAcquisition".
/// Before calling the PrepareAcquisition functions, one call to SetConfigurationSetup is done by the PlugIn.
/// </summary>
/// <param name="ulSystemIdentifier">
///   Identifier, defined on the configuration tab page "Identifer" in IPEmotion
///   This identifier can be used to distinguish different "Serial" systems, if
///   more than one are defined in IPEmotion.
/// </param>
HRESULT EXTENSION_API PrepareAcquisition(ULONG ulSystemIdentifier)
{
	HRESULT RetVal = NO_ERROR;

	// do some prepartion if needed, eg send some start or configuration signals

	return RetVal;
}

///////////////////////////////////////////////////////////////////////////////////////////
/// <summary>
/// Function called on the beginning of measurement timer (cycle).
/// </summary>
/// <param name="ulSystemIdentifier">
///   Identifier, defined on the configuration tab page "Identifer" in IPEmotion
///   This identifier can be used to distinguish different "Serial" systems, if
///   more than one are defined in IPEmotion.
/// </param>
HRESULT EXTENSION_API DoPreAcquisitionTimer(ULONG ulSystemIdentifier)
{
	HRESULT RetVal = NO_ERROR;

	// send some commands via COM port if needed

	return RetVal;
}

///////////////////////////////////////////////////////////////////////////////////////////
/// <summary>
/// Function called on the end of measurement timer (cycle)
/// </summary>
/// <param name="ulSystemIdentifier">
///   Identifier, defined on the configuration tab page "Identifer" in IPEmotion
///   This identifier can be used to distinguish different "Serial" systems, if
///   more than one are defined in IPEmotion.
/// </param>
HRESULT EXTENSION_API DoPostAcquisitionTimer(ULONG ulSystemIdentifier)
{
	HRESULT RetVal = NO_ERROR;

	// read data from COM port until data is complete
	// parse read data for packet values
	// transmit packet data to plugin via data-callback g_pfnDataFloat64

	return RetVal;
}

///////////////////////////////////////////////////////////////////////////////////////////
/// <summary>
/// Measurement finalization.This function is called once for every System in the PlugIn function "FinishAcquisition".
/// </summary>
/// <param name="ulSystemIdentifier">
///   Identifier, defined on the configuration tab page "Identifer" in IPEmotion
///   This identifier can be used to distinguish different "Serial" systems, if
///   more than one are defined in IPEmotion.
/// </param>
HRESULT EXTENSION_API FinishAcquisition(ULONG ulSystemIdentifier)
{
	HRESULT RetVal = NO_ERROR;

	// do some finalizing if needed, eg send som stop signals free memory / resources if needed

	return RetVal;
}

///////////////////////////////////////////////////////////////////////////////////////////

#ifdef __cplusplus
}
#endif
